@@ -1,6 +1,6 @@
 ;;; package-lint-test.el --- Test suite for package-lint
 
-;; Copyright (C) 2016-2017  Steve Purcell, Fanael Linithien
+;; Copyright (C) 2016-2019  Steve Purcell, Fanael Linithien
 
 ;; Author: Steve Purcell <steve@sanityinc.com>
 ;;         Fanael Linithien <fanael4@gmail.com>
@@ -68,12 +68,12 @@ headers and provide form."
   (let ((reserved-message "This key sequence is reserved (see Key Binding Conventions in the Emacs Lisp manual)"))
     ;; C-c and a letter (either upper or lower case)
     (should (equal (package-lint-test--run "(kbd \"C-c n\")")
-                   `((6 13 warning ,reserved-message))))
+                   `((6 13 error ,reserved-message))))
     (should (equal (package-lint-test--run "(local-set-key \"\\C-cF\" 'something)")
-                   `((6 34 warning ,reserved-message))))
+                   `((6 34 error ,reserved-message))))
     (should-not (package-lint-test--run "(kbd \"C-d n\")"))
     (should (equal (package-lint-test--run "(kbd \"C-c x n\")")
-                   `((6 15 warning ,reserved-message))))
+                   `((6 15 error ,reserved-message))))
 
     ;; [C-keyname] bindings should work fine
     (should-not (package-lint-test--run "(define-key map [C-return] 'something)"))
@@ -88,9 +88,9 @@ headers and provide form."
 
     ;; Function keys <F5> through <F9> without modifier keys
     (should (equal (package-lint-test--run "(define-key map (kbd \"<f5>\") 'something)")
-                   `((6 40 warning ,reserved-message))))
+                   `((6 40 error ,reserved-message))))
     (should (equal (package-lint-test--run "(global-set-key [f5] 'something)")
-                   `((6 32 warning ,reserved-message))))
+                   `((6 32 error ,reserved-message))))
     (should-not (package-lint-test--run "(global-set-key [f4] 'something)"))
     (should-not (package-lint-test--run "(global-set-key (kbd \"C-c <tab>\") 'something)"))
 
@@ -103,21 +103,21 @@ headers and provide form."
 
     ;; Don't bind C-h following any prefix character
     (should (equal (package-lint-test--run "(defcustom test-something (kbd \"C-x C-h\"))")
-                   `((6 41 warning ,reserved-message))))
+                   `((6 41 error ,reserved-message))))
     (should-not (package-lint-test--run "(defcustom test-something (kbd \"C-h C-x\"))"))
 
     ;; Don't bind a key sequence ending in <C-g>
     (should (equal (package-lint-test--run "(defcustom test-something (kbd \"C-x C-g\"))")
-                   `((6 41 warning ,reserved-message))))
+                   `((6 41 error ,reserved-message))))
     (should (equal (package-lint-test--run "(global-set-key \"\\C-c\\C-g\" 'something)")
-                   `((6 38 warning ,reserved-message))))
+                   `((6 38 error ,reserved-message))))
     (should-not (package-lint-test--run "(global-set-key \"C-x g\" 'something)"))
     ;; But we allow C-g alone, which is acceptable for some special cases
     (should-not (package-lint-test--run "(defcustom test-something (kbd \"C-g\"))"))
 
     ;; Don't bind a key sequence ending in <ESC> except following another <ESC>
     (should (equal (package-lint-test--run "(defcustom test-something (kbd \"C-x <ESC>\")")
-                   `((6 43 warning ,reserved-message))))
+                   `((6 43 error ,reserved-message))))
     (should-not (package-lint-test--run "(defcustom test-something (kbd \"C-x <ESC> <ESC>\"))"))))
 
 (ert-deftest package-lint-test-error-autoloads-on-private-functions ()
@@ -221,7 +221,7 @@ headers and provide form."
 (ert-deftest package-lint-test-warn-empty-summary ()
   (should
    (equal
-    '((1 0 warning "Package should have a non-empty summary."))
+    '((1 0 error "Package should have a non-empty summary."))
     (package-lint-test--run "" ";;; test.el ---\n"))))
 
 (ert-deftest package-lint-test-warn-too-long-summary ()
@@ -233,6 +233,14 @@ headers and provide form."
      ""
      ";;; test.el --- xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"))))
 
+(ert-deftest package-lint-test-warn-summary-end-with-period ()
+  (should
+   (equal
+    '((1 0 warning "The package summary should not end with a period."))
+    (package-lint-test--run
+     ""
+     ";;; test.el --- A test.\n"))))
+
 (ert-deftest package-lint-test-warn-emacs-in-summary ()
   (should
    (equal
@@ -319,6 +327,22 @@ headers and provide form."
 Alternatively, depend on (emacs \"24.3\") or greater, in which cl-lib is bundled.")
     (package-lint-test--run ";; Package-Requires: ((cl-lib \"1\"))"))))
 
+(ert-deftest package-lint-test-warning-cl ()
+  (should
+   (member
+    '(6 10 warning "Replace deprecated `cl' with `cl-lib'.  The `cl-libify' package can help with this.")
+    (package-lint-test--run "(require 'cl)"))))
+
+(ert-deftest package-lint-test-warning-cl-macs-etc ()
+  (should
+   (member
+    '(6 10 warning "This file is not in the `cl-lib' ELPA compatibility package: require `cl-lib' instead.")
+    (package-lint-test--run "(require 'cl-macs)")))
+  (should
+   (member
+    '(6 10 warning "This file is not in the `cl-lib' ELPA compatibility package: require `cl-lib' instead.")
+    (package-lint-test--run "(require 'cl-seq)"))))
+
 (ert-deftest package-lint-test-accept-normal-deps ()
   (should (equal '() (package-lint-test--run
                       ";; Package-Requires: ((package-lint \"0.2\") (cl-lib \"0.5\"))"))))
@@ -595,6 +619,10 @@ Alternatively, depend on (emacs \"24.3\") or greater, in which cl-lib is bundled
    (equal
     '()
     (package-lint-test--run "(format \"%s\" \"foo\")")))
+  (should
+   (equal
+    '()
+    (package-lint-test--run "'(error . 0)")))
   (should
    (equal
     '()
@@ -635,5 +663,11 @@ Alternatively, depend on (emacs \"24.3\") or greater, in which cl-lib is bundled
     '((7 2 warning "Closing parens should not be wrapped onto new lines."))
     (package-lint-test--run "(hello\n  )    ; foo"))))
 
+(ert-deftest package-lint-test-accept-lonely-parens-with-preceding-comment ()
+  (should
+   (equal
+    '()
+    (package-lint-test--run "(hello 'world\n ;; a comment\n)"))))
+
 (provide 'package-lint-test)
 ;;; package-lint-test.el ends here
